
 UNIT UnL07011 ;

 INTERFACE

 Procedure ChangeDesCartEiler ;

 Procedure ForBackWardEil ;

 IMPLEMENTATION

 Uses
   UnConTyp , { const }
   UnForPrt , { PrtFile }
   UnL01012 , { initial elements }
   UnForKep , { procedures for kepler elements }
   UnForEil , { procedures for eiler  elements }
   UnForFun , { for function DATan2 for example }
   UnHarMod ; { for GemParmValue[1..4] }

 Const
   numt   = 7 ;
   tint   : Array[1..numt] of Extended =  { time interval in day }
            ( 0.0 , 1.0 , 10.0 , 20.0 , 50.0 , 100.0 , 500.0 ) ;

 Var
   difc   : Array[1..numt] of TypeDim3 ;

 Procedure FourInitConst ( Var  c  : TEilerP ) ;
 Var
   i  : Byte ;
 Begin
   For i:=1 To 4 Do  c[i]:=GemParmValue[i]; { fm , r0 , c , sigma }
 End;

 Procedure ToWriteOneE ( c  : TEilerP ) ;
 Var
   q,r  : Extended ;
 Begin
   r:=Sqrt(1-Sqr(c[10]));
   If  c[11] < 0.0  Then  r:=-r;
   q:=RadGra*DATan2(c[10],r); { from UnForFun }
   WriteLn(PrtFile,'    a=',c[8]:30:18,'  km');
   WriteLn(PrtFile,'    e=',c[9]:30:18);
   WriteLn(PrtFile,'    s=',c[10]:30:18,q:17:6,'  degree');
   WriteLn(PrtFile,'alpha=',c[11]:30:18);
   r:=RadGra*86400; { from radian/s to degree/day }
   WriteLn(PrtFile,'  n_l=',(r/360.0)*c[24]:30:18,'  rev. per day');
   WriteLn(PrtFile,'  n_g=',r*c[25]:30:18,'  degree per day');
   WriteLn(PrtFile,'  n_h=',r*c[26]:30:18,'  degree per day');
 End;

 Procedure ToWriteTwoE ( ul , ug , uh  : Extended ) ;
 Begin
   WriteLn(PrtFile,'    l=',RadGra*ul:18:6,'  degree');
   WriteLn(PrtFile,'    g=',RadGra*ug:18:6,'  degree');
   WriteLn(PrtFile,'    h=',RadGra*uh:18:6,'  degree');
 End;

 Procedure ToWriteThree ;
 Var
   nt  : Byte ;
 Begin
   WriteLn(PrtFile,' ':3,'day',' ':16,'dx  km',
                   ' ':16,'dy  km',' ':16,'dz  km');
   For nt:=1 To numt Do
     Begin
       WriteLn(PrtFile,tint[nt]:6:1,difc[nt][1]:22:16,
                       difc[nt][2]:22:16,difc[nt][3]:22:16);
     End;
 End;

 { lecture 7 exercize 1 }

 Procedure ChangeDesCartEiler ;
 Var
   nv,nparm  : Byte ;
   ceil  : TEilerP ;  { from UnForEil }
   elpa,elua,posa,vela  : TypeDim3 ; { from UnConTyp }
   elpb,elub,posb,velb  : TypeDim3 ;
   ul,ug,uh  : Extended ;
 Begin
   FourInitConst(ceil);
   ForInitComp ; { from UnL01012 }
   For nv:=1 To mvar Do
     Begin
       WriteLn(PrtFile);
       WriteLn(PrtFile,'variant',nv:4);
       elpa[1]:=avar[nv];
       elpa[2]:=evar[nv];
       elpa[3]:=vinc[nv];
       elua[1]:=Random(360);
       elua[2]:=Random(360);
       elua[3]:=Random(360); { initial Kepler elements }
       FromKeplerToCart(elpa,elua,posa,vela); { init pos vel from UnForKep }
       FromCartToEiler(posa,vela,ceil,ul,ug,uh); { from UnForEil }
       FromEilerToCart(ceil,ul,ug,uh,posb,velb);
       ToWriteOneE(ceil);
       ToWriteTwoE(ul,ug,uh);
       For nparm:=1 To 3 Do
         WriteLn('position',posa[nparm]:20:9,'  km ',
                 '     dif',posb[nparm]-posa[nparm]:22:16,'  km');
       For nparm:=1 To 3 Do
         WriteLn('velosity',1.0e3*vela[nparm]:20:9,'  m/s',
                 '     dif',
                 1.0e3*(velb[nparm]-vela[nparm]):22:16,'  m/s');
       For nparm:=1 To 3 Do
         WriteLn(PrtFile,'position',posa[nparm]:19:9,'  km ',
                         '     dif',
                         posb[nparm]-posa[nparm]:22:16,'  km');
       For nparm:=1 To 3 Do
         WriteLn(PrtFile,'velosity',1.0e3*vela[nparm]:19:9,'  m/s',
                         '     dif',
                         1.0e3*(velb[nparm]-vela[nparm]):22:16,'  m/s');
     End;
 End;

 Procedure ForBackWardEil ;
 Var
   nv,nt,nparm  : Byte ;
   ul,ug,uh,dt  : Extended ;
   ulc,ugc,uhc  : Extended ;
   ulw,ugw,uhw  : Extended ;
   ceil,weil    : TEilerP ;  { from UnForEil }
   elpa,elua,posa,vela  : TVect3 ; { form UnConTyp }
   elpb,elub,posb,velb  : TVect3 ;
   elpc,eluc,posc,velc  : TVect3 ;
 Begin
   FourInitConst(ceil);
   FourInitConst(weil);
   ForInitComp ; { from UnL01012 }
   For nv:=1 To mvar Do
     Begin
       elpa[1]:=avar[nv]; { a  km }
       elpa[2]:=evar[nv]; { e }
       elpa[3]:=vinc[nv]; { i  degree }
       elua[1]:=Random(360); { Omega  degree }
       elua[2]:=Random(360); { omega  degree }
       elua[3]:=Random(360); { M  degree }
       FromKeplerToCart(elpa,elua,posa,vela); { from UnForKep }
       FromCartToEiler(posa,vela,ceil,ul,ug,uh); { from UnForEil }
       WriteLn(PrtFile);
       WriteLn(PrtFile,'variant',nv:4);
       ToWriteOneE(ceil);
       ToWriteTwoE(ul,ug,uh);
       For nt:=1 To numt Do
         Begin
           dt:=+86400*tint[nt]; { interval in second }
           ulc:=ul+ceil[24]*dt; { l=l_0+n_0/(1-\lambda)*(t-t_0) }
           ugc:=ug+ceil[25]*dt; { g=g_0+n_0*\nu/(1-\lambda)*(t-t_0) }
           uhc:=uh+ceil[26]*dt; { h=h_0+n_0*\mu/(1-\lambda)*(t-t_0) }
           FromEilerToCart(ceil,ulc,ugc,uhc,posc,velc);
           FromCartToEiler(posc,velc,weil,ulw,ugw,uhw); { from UnForEil }
           dt:=-86400*tint[nt]; { interval in second }
           ulc:=ulw+weil[24]*dt; { l=l_0+n_0/(1-\lambda)*(t-t_0) }
           ugc:=ugw+weil[25]*dt; { g=g_0+n_0*\nu/(1-\lambda)*(t-t_0) }
           uhc:=uhw+weil[26]*dt; { h=h_0+n_0*\mu/(1-\lambda)*(t-t_0) }
           FromEilerToCart(weil,ulc,ugc,uhc,posb,velb);
           WriteLn(' pos',posa[1]:20:9,'  km, ',
                   '  dt',tint[nt]:6:1,'  day, ',
                   ' dif',posb[1]-posa[1]:22:16,'  km');
           For nparm:=1 To 3 Do
             difc[nt][nparm]:=posb[nparm]-posa[nparm];
         End;
       ToWriteThree ;
     End;
 End;

 End.